INSERT INTO helios_params_tb (worker_class, host, param, value) 
VALUES ('MP3IndexerService','*','mp3db_dsn','');

INSERT INTO helios_params_tb (worker_class, host, param, value) 
VALUES ('MP3IndexerService','*','mp3db_user','');

INSERT INTO helios_params_tb (worker_class, host, param, value) 
VALUES ('MP3IndexerService','*','mp3db_pass','');

